if (!window.$wzp) {window.$wzp={}}

window.$wzp.openstreetMapL = function (instance, conf){
    if (!instance) return;
    
    const PATH_TO_ROOT = document.querySelector("html").getAttribute("data-ptr") || "";
    
    /* #region Locate-Button */
    if (conf.showLocationBtn) {
        const LocationButton = L.Control.extend({
            onAdd: function(map) {
                const container = L.DomUtil.create("div", "leaflet-bar leaflet-control");
                const btn = L.DomUtil.create("a", "wzp_osmMap__locateBtn leaflet-bar-part leaflet-bar-part-single");

                btn.role = "button";
                btn.title = "Standort anzeigen";
                btn.innerHTML = '<i class="fa fa-location-arrow" aria-hidden="true"></i>';
                btn.addEventListener("click", function () {
                    instance.locate({setView: true, maxZoom: conf.locateMaxZoom});
                });
                
                container.appendChild(btn);

                return container;
            },

            onRemove: function(map) {}
        });
        L.control.locationButton = function(opts) {
            return new LocationButton(opts);
        }

        L.control.locationButton({ position: "topleft" }).addTo(instance);
    }
    /* #endregion */
    
    /* #region Searchbar */
    if (conf.showSearchField) {
        async function getGeo(search, country = "") {
            const f = await fetch(PATH_TO_ROOT + `assets/php/wzp-openstreetmap-l/geocode.php?s=${decodeURIComponent(search) ?? ""}&country=${decodeURIComponent(country)}&lang=${conf.lang}`);
            return f.json();
        }
        
        const SearchBar = L.Control.extend({
            onAdd: function(map) {
                const container = L.DomUtil.create("div", "leaflet-bar leaflet-control");
                const input = L.DomUtil.create("input", "wzp_osmMap__search leaflet-bar-part leaflet-bar-part-single");
                
                input.type = "search";
                input.placeholder = " " + (conf.searchPlaceholder ?? "");
                
                if (conf.minimizeSearch) {
                    input.classList.add("wzp_osmMap__search--minimized");
                }
                
                /* Add datalist */
                const resultsList = document.createElement("ul");
                resultsList.classList.add("wzp_osmMap__searchList");
                resultsList.setAttribute("aria-expanded", "false");
                
                let keyStrokeCount = 0;
                let timeout;
                
                /* Sets the view of the map to show the entry passed as the parameter */
                function show(entry){
                    map.setView([entry.lat, entry.lng], conf.locateMaxZoom);
                    input.blur();
                    input.value = conf.minimizeSearch ? "" : `${entry.name} (${entry.country})`;
                }
                
                function handleChange(e) {
                    if (e.key == "ArrowUp" || e.key == "ArrowDown" || e.key == "ArrowLeft" || e.key == "ArrowRight") return;
                    
                    let search = input.value.trim();
                    let country = "";
                    
                    if (search.includes(" (")) {
                        const split = search.split(" (");
                        search = split[0];
                        country = split[1].replace(")", "");
                    }

                    keyStrokeCount++;

                    if (keyStrokeCount <= 5) {
                        clearTimeout(timeout);
                    } else {
                        keyStrokeCount = 0;
                    }

                    timeout = setTimeout(() => {
                        getGeo(search, country).then((result) => {
                            resultsList.innerHTML = "";
                            
                            if (e.key == "Enter") {
                                /* Show the first result when enter is pressed */
                                show(result.data[0]);
                            } else {    
                                result.data.forEach((entry) => {
                                    const li = document.createElement("li");
                                    const btn = document.createElement("button");
                                    
                                    btn.addEventListener("click", () => show(entry));
                                    
                                    btn.innerHTML = `${entry.name} <span class="wzp_osmMap__searchList__country">(${entry.country})</span>`;
                                    
                                    li.appendChild(btn);
                                    resultsList.appendChild(li);
                                });
                            }
                        });
                    }, 200);
                    
                    resultsList.setAttribute("aria-expanded", "true");
                }
                
                input.addEventListener("keyup", handleChange);
                
                input.addEventListener("change", handleChange);
                
                input.addEventListener("focus", () => {
                    resultsList.setAttribute("aria-expanded", "true");
                });
                
                input.addEventListener("blur", () => {
                    setTimeout(() => {
                        resultsList.setAttribute("aria-expanded", "false");
                    }, 200);
                });
                
                container.appendChild(input);
                container.appendChild(resultsList);

                return container;
            },

            onRemove: function(map) {}
        });
        L.control.searchBar = function(opts) {
            return new SearchBar(opts);
        }
        
        const bar = L.control.searchBar({ position: "topright" }).addTo(instance);
        
        /* Disbale dragging the map when the mouse is over the input field */
        bar.getContainer().addEventListener("mousedown", () => {
            instance.dragging.disable();
        });
        instance.addEventListener("mouseup", () => {
            instance.dragging.enable();
        });
    }
    /* #endregion */
};